import chimera, numpy
from numpy import *

# -----------------------------------------------------------------------------
# For a set of 4 consecutive atoms, compute the dihedral angle.
# The input is a list of atoms.
# The angle is returned in degrees.
def dihedralAngle(atoms_L):
    u = atoms_L[1].coord() - atoms_L[0].coord()
    v = atoms_L[2].coord() - atoms_L[1].coord()
    w = atoms_L[3].coord() - atoms_L[2].coord()
    uCROSSv = tuple([u[1]*v[2]-u[2]*v[1], u[2]*v[0]-u[0]*v[2], u[0]*v[1]-u[1]*v[0]])
    vCROSSw = tuple([v[1]*w[2]-v[2]*w[1], v[2]*w[0]-v[0]*w[2], v[0]*w[1]-v[1]*w[0]])
    uCROSSvN = uCROSSv/linalg.norm(uCROSSv)
    vCROSSwN = vCROSSw/linalg.norm(vCROSSw)
    angle = numpy.sign(numpy.dot(uCROSSvN, w)) * math.acos(dot(uCROSSvN, vCROSSwN))
    return angle*180/math.pi
    
#==============================================================================
# Use a protein with a good selection of beta strands and helices (Eg. 1AON)
pdbIDchars = raw_input("Type in 4 character PDB ID or PDB file name: \n")
prot = chimera.openModels.open(pdbIDchars, type="PDB")[0]

# Data lists (atom names designate the center atom for the bond angle):
# H: helix, S: strand, L: loop.
phiH = []; phiS = []; phiL = []
psiH = []; psiS = []; psiL = []


for r in prot.residues:
    rNxt = prot.residueAfter(r)
    rBfr = prot.residueBefore(r)
    if rNxt and rBfr:
        if rBfr.findAtom("C") and r.findAtom("N") and r.findAtom("CA") and r.findAtom("C") and rNxt.findAtom("N"):
            phi = dihedralAngle([rBfr.findAtom("C"), r.findAtom("N"), r.findAtom("CA"), r.findAtom("C")])
            psi = dihedralAngle([r.findAtom("N"), r.findAtom("CA"), r.findAtom("C"), rNxt.findAtom("N")])
            
            if r.isHelix:
                phiH.append(phi)
                psiH.append(psi)
            if r.isStrand:
                phiS.append(phi)
                psiS.append(psi)
            if not r.isHelix and not r.isStrand:
                phiL.append(phi)
                psiL.append(psi)


print "PHI dihedral angles:"
print "Secondary      Mean      Standard    Max        Min          Sample"
print "Structure:               Deviation                           Size"
print "Helix:   " + 4*"%11.2f" % (array(phiH).mean(), array(phiH).std(), array(phiH).max(), array(phiH).min()), 7*" ", len(phiH)
print "Strand:  " + 4*"%11.2f" % (array(phiS).mean(), array(phiS).std(), array(phiS).max(), array(phiS).min()), 7*" ", len(phiS)
print "Loop:    " + 4*"%11.2f" % (array(phiL).mean(), array(phiL).std(), array(phiL).max(), array(phiL).min()), 7*" ", len(phiL)
print "\n"
            
print "PSI dihedral angles:"
print "Secondary      Mean      Standard    Max        Min          Sample"
print "Structure:               Deviation                           Size"
print "Helix:   " + 4*"%11.2f" % (array(psiH).mean(), array(psiH).std(), array(psiH).max(), array(psiH).min()), 7*" ", len(psiH)
print "Strand:  " + 4*"%11.2f" % (array(psiS).mean(), array(psiS).std(), array(psiS).max(), array(psiS).min()), 7*" ", len(psiS)
print "Loop:    " + 4*"%11.2f" % (array(psiL).mean(), array(psiL).std(), array(psiL).max(), array(psiL).min()), 7*" ", len(psiL)
print "\n"
            
