import chimera, numpy
from numpy import array, cross, dot

def chiralityExaminer(caAtom):
    # Return codes:
    #   L: L-configuration
    #   D: D-configuration    
    #   N: Not applicable (for example, the CA in glycine)
    
    retVal = "N" # (Until proven otherwise).

    n_Atom = caAtom.residue.findAtom('N')
    c_Atom = caAtom.residue.findAtom('C')
    cbAtom = caAtom.residue.findAtom('CB')

    if not(n_Atom and c_Atom and cbAtom): return retVal
    
    vectorTo_N = array(n_Atom.coord()) - array(caAtom.coord())
    vectorTo_C = array(c_Atom.coord()) - array(caAtom.coord())
    vectorTo_B = array(cbAtom.coord()) - array(caAtom.coord())
    crossProd_vN_vC = cross(vectorTo_N, vectorTo_C)

    dotVal = dot(crossProd_vN_vC, vectorTo_B)
    if dotVal > 0.0: return "L"
    else: return "D"



#==============================================================================
# Mainline:
#==============================================================================

pdbID = raw_input("Type in a PDB ID \n")  # For D-configs try: 1AN1, 1PPF, 1NRQ, 1GOE, 1DSR, 2SOC
prot = chimera.openModels.open(pdbID, type = 'PDB')[0]

for r in prot.residues:
    if r.findAtom('CA'):
        print r.id.position, r.type, chiralityExaminer(r.findAtom('CA'))
