import chimera, numpy
from numpy import array

from StructBio.LeastSquares.planes import LeastSquaresPlane
from StructBio.LeastSquares.inertialAxes import InertialAxes

def calculateHelixAngles(prot):
    # Collect backbone atoms in both the E and F helices:
    # Get N atoms from the HEM residue as well.
    helix_E_atoms_L = []
    helix_F_atoms_L = []
    beg_E_position = 60
    end_E_position = 77
    beg_F_position = 86
    end_F_position = 95
    hemeNatoms_L = []
    for r in prot.residues:
        if r.id.position == beg_E_position: beg_E_atom = r.findAtom('N')
        if r.id.position == end_E_position: end_E_atom = r.findAtom('C')                                                  
        if r.id.position >= beg_E_position and r.id.position <= end_E_position:
            helix_E_atoms_L.append(r.findAtom('N'))
            helix_E_atoms_L.append(r.findAtom('CA'))
            helix_E_atoms_L.append(r.findAtom('C'))
            helix_E_atoms_L.append(r.findAtom('O'))
        if r.id.position == beg_F_position: beg_F_atom = r.findAtom('N')
        if r.id.position == end_F_position: end_F_atom = r.findAtom('C')
        if r.id.position >= beg_F_position and r.id.position <= end_F_position:
            helix_F_atoms_L.append(r.findAtom('N'))
            helix_F_atoms_L.append(r.findAtom('CA'))
            helix_F_atoms_L.append(r.findAtom('C'))
            helix_F_atoms_L.append(r.findAtom('O'))
        if r.type == "HEM":
            hemeNatoms_L.append(r.findAtom('NA'))
            hemeNatoms_L.append(r.findAtom('NB'))
            hemeNatoms_L.append(r.findAtom('NC'))
            hemeNatoms_L.append(r.findAtom('ND'))
            hemeFe_atom = r.findAtom('FE')

    inertial_E_axis = InertialAxes(helix_E_atoms_L)
    inertial_F_axis = InertialAxes(helix_F_atoms_L)

    # Get two points on each axis:
    beg_E_axis = inertial_E_axis.projectionOnAxis(beg_E_atom)
    end_E_axis = inertial_E_axis.projectionOnAxis(end_E_atom)
    beg_F_axis = inertial_F_axis.projectionOnAxis(beg_F_atom)
    end_F_axis = inertial_F_axis.projectionOnAxis(end_F_atom)

    hemePlane = LeastSquaresPlane(hemeNatoms_L)

    angle_E_helix = hemePlane.angleToPlane(beg_E_axis, end_E_axis)
    angle_F_helix = hemePlane.angleToPlane(beg_F_axis, end_F_axis)
    dist_FeToPlane = hemePlane.distanceToPlane(hemeFe_atom)
    return (angle_E_helix, angle_F_helix, dist_FeToPlane)
    
#==============================================================================
# Mainline:
#==============================================================================

mbCO_data_T = calculateHelixAngles(chimera.openModels.open("1BZR", type = 'PDB')[0])
deOxydata_T = calculateHelixAngles(chimera.openModels.open("1BZP", type = 'PDB')[0])

print ("mbCO:  E and F helix angles and distance Fe to heme plane: %6.3f  %6.3f   %6.3f" % mbCO_data_T)
print ("deOXY: E and F helix angles and distance Fe to heme plane: %6.3f  %6.3f   %6.3f" % deOxydata_T)
print ("                                             Delta values: %6.3f  %6.3f   %6.3f" % tuple(array(mbCO_data_T) - array(deOxydata_T)))

